/* lunghezza massima delle stringhe chiave (key) */
#define MAXL 256

      
/* tipo del valore associato alla chiave */
typedef int typevalue_t;

/* tipo elemento della hash table */
typedef struct hashE{                                    
    char * key;           /* chiave NULL se elemento vuoto*/
    typevalue_t value;    /* valore associato a key */
    struct hashE * next;  
} hashElement_t;


/* tipo della tabella hash  (un array di puntatori a hashElement_t) */
typedef struct { 
    int size;    /* ampiezza tabella */
    hashElement_t ** table; 
} hashTable_t;

/* FUNZIONI DA IMPLEMENTARE */
/* @description -- crea la labella hash e la inizializza
   @param  sizeH -- numero di elementi nella tabella 
                      (meglio scegliere un numero primo!) 
   @returns -- (puntatore alla nuova hash table) esecuzione corretta 
               (NULL) situazioni erronee 
*/
hashTable_t * createHash(int sizeH);        

/* @description -- cancella tutta la tabella hash liberando la memoria
   @param  ht  -- tabella da cancellare
*/
void deleteHash(hashTable_t * ht);        

/* @description -- hashFunction
   @param s -- stringa su cui calcolare l'hash
   @param ht -- ampiezza della HashTable
   @returns -- (-1) si sono verificati errori
               ( 0 <= n < (ht->size) ) valore dell'hash
*/
int hashFunction (const char* s, hashTable_t* ht);     

/* @description -- inserisce (key, valore) nell' hash table se non c'e' gia'
   @param s -- key 
   @param value -- valore
   @param ht -- puntatore alla hashTable
   @returns -- (NULL) si sono verificati errori (elemento gia' presente etc)
               (p) puntatore all'elemento inserito
*/
hashElement_t* putHash(hashTable_t* ht, const char * s, typevalue_t value);     

/* @description -- cerca un elemento nell'hash
   @param s -- chiave
   @param ht -- puntatore alla hashTable
   @returns -- (NULL) si sono verificati errori
               (p) puntatore all'elemento con chiave s
*/
hashElement_t* getHash(hashTable_t* ht, const char* s);

/* @description -- rimuove un elemento dall'hash
   @param s -- chiave
   @param ht -- puntatore alla hashTable
   @returns -- (NULL) si sono verificati errori
               (p) puntatore all'elemento rimosso
*/
hashElement_t* removeFromHash (hashTable_t* ht, const char* s);

/* @description -- stampa formattata di un elemento dell'hash
   @param e -- puntatore elemento da stampare 
   @returns -- (-1) si sono verificati errori
               (n>=0) se OK
*/
int printHashElement (FILE * uscita, hashElement_t * e);

/* @description -- dealloca la memoria di un elemento
   @param e -- puntatore elemento da deallocare
*/
void deleteHashElement (hashElement_t * e);





